/****************************************************************
**
** Implementation of Gantt class
**
** This software is released under the GPL General Public License,
** see COPYING for details.
**
****************************************************************/

#include <iostream.h>

#include "gantt.h"

#include <qpainter.h>
#include <qprinter.h>
#include <qprintdialog.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qevent.h>


Gantt::Gantt( char *filenamearg, QWidget *parent, const char *name )
        : QWidget( parent, name )
	{
	setBackgroundColor( "white" );
	textpixx = 150;
	daypix = 10;
	fileisread = 0;
	isprinting = 0;
	weekdetail = 0;
	numtasks = 0;
	changed = 0;
	TEXTPIXY = 50;
	file = new QFile();
	printer = new QPrinter();
	file->setName( filenamearg );
	if( file->exists() )
		{
		open( file );
		repaint();
		}
	}


void Gantt::reset()
{
	int t;
	if ( changed )
	{
		switch (QMessageBox::information( this, "QtGantt",
				"This file contains unsaved work\n"
				"Do you want to save it before creating a new file?\n",
				"&Save", "&Don't save", "&Cancel", 0, 2))
		{
		case 0: // Save
			save();
			break;
		case 1: // Don't Save
			break;
		case 2: // Cancel
			return;
		}
	}
	for (t=0; t<numtasks; t++)
	{
		delete tasks[t]->type;
		delete tasks[t]->startdate;
		if (tasks[t]->enddate) delete tasks[t]->enddate;
		delete tasks[t]->description;
		delete tasks[t];
	}
	numtasks = 0;
	changed = 0;
	repaint();
}

void Gantt::save()
{
	QMessageBox::information( this, "QtGantt",
				"Save not currently implemented\n");
}

void Gantt::open()
	{
	QString filename( QFileDialog::getOpenFileName( 0, "*.gantt" ) );
	if ( !filename.isNull() )             // if cancel was not pressed
		{
		file->setName( (const char *)filename );
		open( file );
		}
	}


void Gantt::print()  // draw gantt to printer
	{
	if( fileisread )    // if file reading went OK
		{
		if ( printer->setup() )   // if pushed OK button in printer setup
			{
			isprinting = 1;
			QPainter painter;
			painter.begin( printer );
			paint( &painter );      // calls the paint function
			painter.end();
			isprinting = 0;
			}
		}
	}

void Gantt::printToFile( char * filename )  // draw gantt to file without setup dialog
	{
	if( fileisread )    // if file reading went OK
		{
			isprinting = 1;
			QPainter painter;
			printer->setOutputFileName( filename );
			painter.begin( printer );
			paint( &painter );      // calls the paint function
			painter.end();
			isprinting = 0;
		}
	}


void Gantt::draw()  // draw gantt to screen
	{
	if( fileisread )   // if file reading went OK
		{
		QPainter painter;
		painter.begin( this );
		paint( &painter );
		painter.end();
		}
	}


void Gantt::refresh()           // when F5 is pushed
	{
	if( file->exists() )
		{
		open( file );
		repaint();
		}
	}


void Gantt::changetextpix( int tpix )
	{
	textpixx = tpix;
	repaint();
	}


void Gantt::changedaypix( int dpix )
	{
	daypix = dpix;
	repaint();
	}

void Gantt::setweekdetail( int newweekdetail )
	{
printf("diag: take changed = 1 out of gantt::setweekdetail\n");
changed = 1;
	weekdetail = newweekdetail;
	repaint();
	}


void Gantt::open( QFile *file )
	{
	if( file->open(IO_ReadOnly) )
		{
		QTextStream t( file );
		readfile( &t );
		fileisread = 1;
		}
	else
		{
		printf( "File not found\n" );
		fileisread = 0;
		}
	}


void Gantt::readfile( QTextStream *t )  // reads gantt file, used by print() and draw()
	{
	QString qstr1, qstr2, yyyy, mm, dd;
	numtasks=0;
	int pos1, pos2, pos3;
	
	while ( !t->eof() && numtasks<NUMTASKS )     // loops through the file
		{
		tasks[numtasks] = new task;
		qstr1 = t->readLine();
		
		if( !qstr1.isEmpty() && qstr1.left(1) != "#" )  // if not an empty line and not a comment
			{
			if( qstr1.left(1) == "%" )    // if a header line
				{
				pos1 = qstr1.find( '%', 1, TRUE );
				qstr2 = qstr1.mid( 1, pos1-1 );
				
				pos2 = qstr2.find( '-', 0, TRUE );
				dd = qstr2.left(pos2);
				pos3 = qstr2.find( '-', pos2+1, TRUE );
				mm = qstr2.mid( pos2+1, pos3-pos2-1 );
				yyyy = qstr2.right( qstr2.length() - pos3-1 );
				project_startdate = new QDate( yyyy.toInt(), mm.toInt(), dd.toInt() );
				
				qstr2 = qstr1.right( qstr1.length() - pos1 - 1 );
				header = new QString( qstr2 );
				}
			else                   // if a task line
				{
				// num
				pos1 = qstr1.find( ':', 0, TRUE );
				qstr2 = qstr1.left(pos1);
				tasks[numtasks]->num = qstr2.toInt();
				
				// type
				pos2 = qstr1.find( ':', pos1+1, TRUE );
				qstr2 = qstr1.mid( pos1+1, pos2-pos1-1 );
				tasks[numtasks]->type = new QString( qstr2 );
				
				// startdate
				pos1 = qstr1.find( ':', pos2+1, TRUE );
				qstr2 = qstr1.mid( pos2+1, pos1-pos2-1 );
				pos3 = qstr2.find( '-', 0, TRUE );
				dd = qstr2.left(pos3);
				pos2 = qstr2.find( '-', pos3+1, TRUE );
				mm = qstr2.mid( pos3+1, pos2-pos3-1 );
				yyyy = qstr2.right( qstr2.length() - pos2-1 );
				tasks[numtasks]->startdate = new QDate( yyyy.toInt(), mm.toInt(), dd.toInt() );
				
				// enddate
				pos2 = qstr1.find( ':', pos1+1, TRUE );
				if( tasks[numtasks]->type->copy() != "mile" )  // if not a milestone
					{
					qstr2 = qstr1.mid( pos1+1, pos2-pos1-1 );
					pos3 = qstr2.find( '-', 0, TRUE );
					dd = qstr2.left(pos3);
					pos1 = qstr2.find( '-', pos3+1, TRUE );
					mm = qstr2.mid( pos3+1, pos1-pos3-1 );
					yyyy = qstr2.right( qstr2.length() - pos1-1 );
					tasks[numtasks]->enddate = new QDate( yyyy.toInt(), mm.toInt(), dd.toInt() );
					}
				else         // milestones don't need enddate, so a blank entry is ok
					tasks[numtasks]->enddate = new QDate( tasks[numtasks]->startdate->addDays(0) );
				
				// after
				pos1 = qstr1.find( ':', pos2+1, TRUE );
				qstr2 = qstr1.mid( pos2+1, pos1-pos2-1 );
				tasks[numtasks]->after = qstr2.toInt();
				
				// percentage copleted
				pos2 = qstr1.find( ':', pos1+1, TRUE );
				qstr2 = qstr1.mid( pos1+1, pos2-pos1-1 );
				tasks[numtasks]->completed = qstr2.toInt();
				
				// description
				qstr2 = qstr1.right( qstr1.length() - pos2-1 );
				tasks[numtasks]->description = new QString( qstr2 );
				
				numtasks++;
				}
			}
		}
	file->close();
	}


void Gantt::paint( QPainter *painter )  // draw gantt, used by print() and draw()
	{
	int page=1, pagewidth, pageheight;
	int textHeight, ascent, twoDigitWidth, leading;
	int canFitDayNums = 0;
	if( isprinting )
		{
//printf("*** PRINTING ***\n");
		// CHANGE: this ain't gonna work - we need to know exactly how
		// big our pages are, so we can calculate pagetasks (before main
		// loop). The Qt book has something about printing.
		pagewidth = 1200;
		pageheight = 800;
		}
	else
		{
//printf("*** DISPLAYING ***\n");
		pagewidth = width();
		pageheight = height();
		}
	
	// Draws the header text
	painter->setPen( "black" );
	painter->setFont( QFont("times", 20, QFont::Bold) );
	painter->drawText( 10, 0, pagewidth-20, TITLEPIXY, AlignCenter, header->copy() );
	QFontMetrics qfm = painter->fontMetrics();
	ascent = qfm.ascent();
	textHeight = ascent + qfm.descent();

	// Draws lines for weeks, months and current date
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal) );
	qfm = painter->fontMetrics();
	ascent = qfm.ascent();
	textHeight = ascent + qfm.descent();
	twoDigitWidth = qfm.width("33", 2);
	leading = qfm.leading();

	// If we're doing week detail, the calendar header is taller, to take into account 
	// the start-of-week day numbers (but only if they'll fit)
	if ( weekdetail && daypix > (twoDigitWidth+8) )
		{
		TEXTPIXY = TITLEPIXY + 10 + (textHeight * 2);
		canFitDayNums = 1;
		}
	else
		{
		TEXTPIXY = TITLEPIXY + 10 + textHeight;
		canFitDayNums = 0;
		}


	int i=0, pagetasks=20, currentpage;
	pagetasks = (pageheight-TEXTPIXY) / SPACING;
//printf("pageheight is %d, pagetasks is %d\n", pageheight, pagetasks);
	while( i<numtasks )
		{
		currentpage = page;

//printf("Drawing headings...\n");
		// Draw top border around months and optional start-of-week day numbers
		painter->setPen( "darkgrey" );
		painter->drawLine( 0, TITLEPIXY, pagewidth, TITLEPIXY );
		painter->drawLine( 0, TEXTPIXY, pagewidth, TEXTPIXY );

		// Draws lines for weeks, months and current date
		painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal) );
		//QFontMetrics qfm = painter->fontMetrics();
		//ascent = qfm.ascent();
		//textHeight = ascent + qfm.descent();
		QDate tempdate = project_startdate->addDays(0);
		for( int day=textpixx; day < pagewidth; day = day+daypix )
			{
			// fill in weekends with light grey
			if( weekdetail && (tempdate.dayOfWeek() == 6 || tempdate.dayOfWeek() == 7))
				{
				painter->setPen ( "lightgrey" );
				painter->setBrush ( "lightgrey" );
				painter->drawRect( day, TEXTPIXY+1, daypix, pageheight);
				}
			// indicate Mondays with a grey line
			if( weekdetail || tempdate.dayOfWeek() == 1 )  // if a monday
				{
				QPen dottedpen( weekdetail ? "black" : "lightgrey", 0, DashLine );
				painter->setPen ( dottedpen );
				painter->drawLine( day, TEXTPIXY, day, pageheight );  // Draws a dotted line for each week
				// display the day number for the start of each
				// week
				if ( canFitDayNums && tempdate.dayOfWeek() == 1 ) // weekdetail is implied by canFitDayNums
					{
					painter->setPen( "black" );
					QString qstr;
					int dayNoY = TITLEPIXY + 5 + textHeight + ascent + leading;
					painter->drawText( day+5, dayNoY, qstr.setNum(tempdate.day()));
					}
				}
			if( tempdate.day() == 1 )    // if first day of month
				{
				painter->setPen( "darkgrey" );
				painter->drawLine( day, TITLEPIXY, day, pageheight );  // Draws a solid line for each month
				painter->setPen( "black" );
				int monthY = TITLEPIXY + 5 + ascent;
				switch( tempdate.month() )
					{
					// these drawTexts are baseline-relative.
					case 1: painter->drawText( day+5, monthY, "January"); break;
					case 2: painter->drawText( day+5, monthY, "February"); break;
					case 3: painter->drawText( day+5, monthY, "March"); break;
					case 4: painter->drawText( day+5, monthY, "April"); break;
					case 5: painter->drawText( day+5, monthY, "May"); break;
					case 6: painter->drawText( day+5, monthY, "June"); break;
					case 7: painter->drawText( day+5, monthY, "July"); break;
					case 8: painter->drawText( day+5, monthY, "August"); break;
					case 9: painter->drawText( day+5, monthY, "September"); break;
					case 10: painter->drawText( day+5, monthY, "October"); break;
					case 11: painter->drawText( day+5, monthY, "November"); break;
					case 12: painter->drawText( day+5, monthY, "December"); break;
					}
				}
			if( tempdate == tempdate.currentDate() )    // if current date
				{
				painter->setPen( "red" );
				painter->drawLine( day, TEXTPIXY+1, day, pageheight );  // Draws a solid red line for the current date
				if ( weekdetail)
					{
					painter->setBrush ( "pink" );
					painter->drawRect( day, TEXTPIXY+1, daypix, pageheight);
					}
				}
			tempdate = tempdate.addDays(1);
			}
		
		// Draws the tasks, mains and milestones
		for( i; i<numtasks && page==currentpage; i++ )  // for each task
			{
//printf("top of loop i is %d, page is %d, currentpage is %d, i%%pagetasks is %d\n", i, page, currentpage, i%pagetasks);

			// Switch to a new page, if we've filled this one (only
			// when printing)
			if( i!=0 && (i%pagetasks) == 0 )
				{
//printf("new page\n");
				if ( isprinting )
					printer->newPage();
				page++;
				}

			if( page == currentpage && tasks[i] )   // if this tasknumber is read from file
				{
//printf("drawing task\n");
				int j=0;
				if( tasks[i]->after != 0 )	// if 'after' is specified (which means the task depends on other tasks)
					{			// search for the last other task
					while( tasks[j]->num != tasks[i]->after && j < i )
						{ j++; }
					}
				
//printf("i is %d, j is %d\n", i, j);
				int jbegin, jend;
				if( (int)( (float)(i/pagetasks)-(float)(j/pagetasks) ) > 0 )
					{
					jbegin = 0;
					jend = 1200;
					}
				else
					{
					jbegin = j%pagetasks;
					jend = j%pagetasks;
					}
//printf("jbegin is %d, jend is %d\n", jbegin, jend);
				
				if( tasks[i]->type->copy() == "task" )   // draws a normal task
					{
					painttask( painter, i, i%pagetasks );
					if( tasks[i]->after != 0 )                     // if 'after' is specified (which means the task depends on other tasks)
						{ painttaskendarrow( painter, i, i%pagetasks, jend ); }   // draws first part of the dependency arrow
					}
				else if( tasks[i]->type->copy() == "main" )   // draws a main task
					{
					paintmain( painter, i, i%pagetasks );
					if( tasks[i]->after != 0 )                     // if 'after' is specified (which means the task depends on other tasks)
						{ paintmainendarrow( painter, i, i%pagetasks, jend ); }   // draws first part of the dependency arrow
					}
				else if( tasks[i]->type->copy() == "mile" )  // draws a milestone
					{
					paintmile( painter, i, i%pagetasks );
					if( tasks[i]->after != 0 )                     // if 'after' is specified (which means the task depends on other tasks)
						{ paintmileendarrow( painter, i, i%pagetasks, jend ); }   // draws first part of the dependency arrow
					}
				
				if( tasks[j]->type->copy() == "task" && tasks[i]->after != 0 )   // draws a normal task
					{
					painttaskbeginarrow( painter, i, i%pagetasks, jbegin );   // draws first part of the dependency arrow
					}
				else if( tasks[j]->type->copy() == "main" && tasks[i]->after != 0 )   // draws a main task
					{
					paintmainbeginarrow( painter, i, i%pagetasks, jbegin );   // draws first part of the dependency arrow
					}
				else if( tasks[j]->type->copy() == "mile" && tasks[i]->after != 0 )  // draws a milestone
					{
					paintmilebeginarrow( painter, i, i%pagetasks, jbegin );   // draws first part of the dependency arrow
					}
				}
//printf("drawing grid\n");

			// draw the grid line
			painter->setPen( "darkgrey" );
			painter->drawLine( 0, TEXTPIXY+((i+1)%pagetasks)*SPACING, pagewidth, TEXTPIXY+((i+1)%pagetasks)*SPACING );


			}
		}
	}


void Gantt::paintEvent( QPaintEvent *e )  // repaints automagically when needed
	{
	draw();
	}


void Gantt::painttask( QPainter *painter, int taskno, int y )
	{
	int taskX, taskY, taskW;
	painter->setPen( "black" );
	painter->setBrush( "lightblue" );

	// Write description
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal) );
	painter->drawText( 10, TEXTPIXY+13+y*SPACING,
	                   textpixx-10, 20, AlignLeft,
	                   tasks[taskno]->description->copy() );

	// Draw rectangle; take percentage completed into account
	taskX = textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0));
	taskY = TEXTPIXY+10+y*SPACING;
	// this is the full width
	taskW = daypix * tasks[taskno]->startdate->daysTo( tasks[taskno]->enddate->addDays(0));
	painter->drawRect( taskX, taskY, taskW, 15);
	if ( tasks[taskno]->completed > 0)
		{
		painter->setBrush( "blue" );
		painter->setPen( "blue" );
		int taskWcomp = (taskW * tasks[taskno]->completed) / 100;
		painter->drawRect( taskX+1, taskY+1, taskWcomp-2, 13);
		}
	}


void Gantt::paintmain( QPainter *painter, int taskno, int y )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Bold) );
	painter->drawText( 10, TEXTPIXY+13+y*SPACING,
	                   textpixx-10, 20, AlignLeft,
	                   tasks[taskno]->description->copy() );
	QPointArray a;
	a.setPoints( 8,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+15+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->enddate->addDays(0) ) + 5,
	             TEXTPIXY+15+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->enddate->addDays(0) ) + 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->enddate->addDays(0) ),
	             TEXTPIXY+25+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->enddate->addDays(0) ) - 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+25+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+20+y*SPACING );
	painter->drawPolygon( a );
	}


void Gantt::paintmile( QPainter *painter, int taskno, int y )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal, 1 ) );
	painter->drawText( 10, TEXTPIXY+13+y*SPACING,
	                   textpixx-10, 20, AlignLeft,
	                   tasks[taskno]->description->copy() );
	QPointArray a;
	a.setPoints( 4,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+15+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+25+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+20+y*SPACING );
	painter->drawPolygon( a );
	QString qstr;
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Bold) );
	painter->drawText( textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) + 10,
	                   TEXTPIXY+25+y*SPACING,
	                   qstr.setNum( tasks[taskno]->startdate->day() ) + "." + qstr.setNum( tasks[taskno]->startdate->month() ) );
	}


void Gantt::painttaskbeginarrow( QPainter *painter, int taskno, int y, int j )
	{
	if ( tasks[j]->enddate->daysTo( tasks[taskno]->startdate->addDays(0) ) * daypix < 7 )
		{
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+17+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	else
		{
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	}


void Gantt::painttaskendarrow( QPainter *painter, int taskno, int y, int j )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+27+j*SPACING,
	                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+3+y*SPACING );
	QPointArray a;
	a.setPoints( 3,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+4+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+4+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+9+y*SPACING );
	painter->drawPolygon( a );
	}


void Gantt::paintmainbeginarrow( QPainter *painter, int taskno, int y, int j )
	{
	if ( tasks[j]->enddate->daysTo( tasks[taskno]->startdate->addDays(0) ) * daypix < 7 )
		{
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+26+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	else
		{
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+18+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+18+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+18+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	}


void Gantt::paintmainendarrow( QPainter *painter, int taskno, int y, int j )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+27+j*SPACING,
	                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+8+y*SPACING );
	QPointArray a;
	a.setPoints( 3,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+9+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+9+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+14+y*SPACING );
	painter->drawPolygon( a );
	}


void Gantt::paintmilebeginarrow( QPainter *painter, int taskno, int y, int j )
	{
	if ( tasks[j]->enddate->daysTo( tasks[taskno]->startdate->addDays(0) ) * daypix < 7 )
		{
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+20+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	else
		{
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING );
		painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	}


void Gantt::paintmileendarrow( QPainter *painter, int taskno, int y, int j )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->drawLine( textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+27+j*SPACING,
	                   textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+7+y*SPACING );
	QPointArray a;
	a.setPoints( 3,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+8+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+8+y*SPACING,
	             textpixx + daypix * project_startdate->daysTo( tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+13+y*SPACING );
	painter->drawPolygon( a );
	}
